#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include "resource.h"
#include "algo.h"

BOOL DoClassRegistration( HINSTANCE hInstance )
{
	WNDCLASS   wndclass;
	memset( &wndclass, 0x00, sizeof( WNDCLASS ) );                           
	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = MainWindowProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = LoadIcon( NULL, IDI_APPLICATION );
	wndclass.hCursor = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = ( HBRUSH )( COLOR_WINDOW + 1 );
	wndclass.lpszMenuName = MAKEINTRESOURCE( IDR_MENU );
	wndclass.lpszClassName = "AlgoSample";
	return RegisterClass( &wndclass );
}
           
BOOL InitInstance( HANDLE hInstance, int nCmdShow )
{
    HWND hWnd;
    hAppInst = hInstance;
    hWnd = CreateWindow( "AlgoSample", "Algorithm Sample Application",
						WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT,
						CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL );
    if( !hWnd ) return 0;
    ShowWindow( hWnd, nCmdShow );
    UpdateWindow( hWnd );
    return 1;
}

int FAR PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
    MSG Message;

    if( !hPrevInstance && !DoClassRegistration( hInstance ) )
        return 0;

    if( !InitInstance( hInstance, nCmdShow ) )
        return 0;

    while( GetMessage( &Message, NULL, 0, 0 ) )
    {
        TranslateMessage( &Message );
        DispatchMessage( &Message );
    }
    return Message.wParam;
}
 
long FAR PASCAL __export MainWindowProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam )
{
	HDC hDC;
	PAINTSTRUCT ps;     
	HMENU hMenu;
	
	switch ( Message ) 
	{
		case WM_PAINT: 
			hDC = BeginPaint( hWnd, &ps );
			SetBkMode( hDC, TRANSPARENT );
			EndPaint( hWnd, &ps );
			break;
		case WM_COMMAND:
			hMenu = GetMenu( hWnd );
			switch ( wParam ) 
			{
				case ID_ERATOSTHENES_STATARRAY:
				{	
					int i;
					char String[50];
					double Time;
					DWORD MiliSecTime = GetTickCount();
					for( i = 0; i < 1000; i++ )
						EratosthenesStaticArray();
					MiliSecTime = GetTickCount() - MiliSecTime;
					Time = ( double ) MiliSecTime / 1000.;
					sprintf( String, "Time of execution: %6.2f miliseconds", Time ); 
					MessageBox( hWnd, String, "Sieve of Eratosthenes [Array]", MB_OK );
					return 0;
				}
				case ID_ERATOSTHENES_DINARRAY:
				{	
					int i;
					char String[50];
					double Time;
					DWORD MiliSecTime = GetTickCount();
					for( i = 0; i < 1000; i++ )
						EratosthenesDinamicArray();
					MiliSecTime = GetTickCount() - MiliSecTime;
					Time = ( double ) MiliSecTime / 1000.;
					sprintf( String, "Time of execution: %6.2f miliseconds", Time ); 
					MessageBox( hWnd, String, "Sieve of Eratosthenes [Array]", MB_OK );
					return 0;
				}
				case ID_ERATOSTHENES_LIST:
				{
					int i;
					char String[50];
					double Time;
					DWORD MiliSecTime = GetTickCount();
					for( i = 0; i < 100; i++ )
						EratosthenesList();
					MiliSecTime = GetTickCount() - MiliSecTime;
					Time = ( double ) MiliSecTime / 100.;
					sprintf( String, "Time of execution: %6.2f miliseconds", Time );
					MessageBox( hWnd, String, "Sieve of Eratosthenes [Linked list]", MB_OK );
					return 0;
				}
				case ID_JOSEPHUS_ARRAY:
				{
					int i;
					char String[50];
					double Time;
					DWORD MiliSecTime = GetTickCount();
					for( i = 0; i < 1000; i++ )
						JosephusArray();
					MiliSecTime = GetTickCount() - MiliSecTime;
					Time = ( double ) MiliSecTime / 1000.;
					sprintf( String, "Time of execution: %6.2f miliseconds", Time );
					MessageBox( hWnd, String, "Josephus problem [Array]", MB_OK );
					return 0;
				}
				case ID_JOSEPHUS_LIST:     
				{
					int i;
					char String[50];
					double Time;
					DWORD MiliSecTime = GetTickCount();
					for( i = 0; i < 1000; i++ )
					JosephusList();
					MiliSecTime = GetTickCount() - MiliSecTime;
					Time = ( double ) MiliSecTime / 1000.;
					sprintf( String, "Time of execution: %6.2f miliseconds", Time );
					MessageBox( hWnd, String, "Josephus problem [Linked list]", MB_OK );
					return 0;
				}
			}
            break;
		case WM_DESTROY :
			PostQuitMessage( 0 );
			return 0;
		default:
			return DefWindowProc( hWnd, Message, wParam, lParam );
	}
	return 0;
}

void EratosthenesStaticArray()
{
	int Array[N+1], i, j;
	for( i = 1; i <= N; i++ ) Array[i] = 1;
	for( i = 2; i <= N/2; i++ )
		if( Array[i] )
			for( j = 2; j <= N/i; j++ )
				Array[i*j] = 0;
}

void EratosthenesDinamicArray()
{
	int* Array, i, j;
	Array = ( int* ) malloc( ( N + 1 ) * sizeof( int ) );
	for( i = 1; i <= N; i++ ) *( Array + i ) = 1;
	for( i = 2; i <= N/2; i++ )
		if( *( Array + i ) )
			for( j = 2; j <= N/i; j++ )
				*( Array + i * j ) = 0;
	free( Array );
}

void EratosthenesList()
{
	int i, j, k;
	LPELEMENT Head, Pointer, Temp;
	
	Head = ( LPELEMENT ) malloc( sizeof( ELEMENT ) );
	Head->Number = 4;
	Head->Next = NULL;
		
	for( i = 2; i <= N/2; i++ )
	{
		Pointer = Head;
		for( j = 2; j <= N/i; j++ )
		{
			k = i * j;
			while( Pointer->Next && Pointer->Number < k && Pointer->Next->Number <= k )
				Pointer = Pointer->Next;
			if( Pointer->Number != k )
			{
				Temp = ( LPELEMENT ) malloc( sizeof( ELEMENT ) );
				Temp->Number = k;
				Temp->Next = Pointer->Next;
				Pointer->Next = Temp;
			}
		}         
	}
	while( Head->Next )
	{
		Temp = Head;
		Head = Head->Next;
		free( Temp );
	}
	free( Head );
}

void JosephusArray()
{                
	int Array[P], i, j = P, ZeroCount = 0;
	for( i = 0; i < P; i++ ) Array[i] = 1;
	while( ( P - 1 ) > ZeroCount++  )
		for( i = 1; i <= M; i++ )
		{
			while( !Array[j%P] ) j++;
			if( i == M ) Array[j%P] = 0;
			j = ++j % P;
		}	
}

void JosephusList()
{
	int i;
	LPELEMENT List, Temp;
	List = ( LPELEMENT ) malloc( sizeof( ELEMENT ) );
	List->Number = 1; Temp = List;
	for( i = 2; i <= P; i++ )
	{
		List->Next = ( LPELEMENT ) malloc( sizeof( ELEMENT ) );
		List = List->Next;
		List->Number = i;
	}                 
	List->Next = Temp;
	while( List != List->Next )
	{
		for( i = 1; i < M; i++ ) List = List->Next;
		Temp = List->Next;
		List->Next = List->Next->Next;
		free( Temp );
	}             
	free( List );
}


           